# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"get_file_paths": "00_html_parsing.ipynb",
         "RE_HTM": "00_html_parsing.ipynb",
         "collect_sheet_uris": "00_html_parsing.ipynb",
         "RE_SHEET": "00_html_parsing.ipynb",
         "download_sheet": "00_html_parsing.ipynb",
         "download_multiple_sheets": "00_html_parsing.ipynb",
         "RE_FNAME": "00_html_parsing.ipynb",
         "is_date": "00_html_parsing.ipynb",
         "get_sheet_df": "00_html_parsing.ipynb",
         "handle_title_and_date": "00_html_parsing.ipynb",
         "disambiguate_party": "00_html_parsing.ipynb",
         "VOTE_COLS": "00_html_parsing.ipynb",
         "PARTY_MAP": "00_html_parsing.ipynb",
         "get_squished_dataframe": "00_html_parsing.ipynb",
         "set_sheet_dtypes": "00_html_parsing.ipynb",
         "DTYPES": "00_html_parsing.ipynb",
         "get_multiple_sheets_df": "00_html_parsing.ipynb",
         "get_multiple_sheets": "00_html_parsing.ipynb",
         "pd.options.plotting.backend": "02_gui.ipynb",
         "get_votes_by_party": "01_similarities.ipynb",
         "prepare_votes_of_mdb": "01_similarities.ipynb",
         "align_mdb_with_parties": "01_similarities.ipynb",
         "compute_similarity": "01_similarities.ipynb",
         "get_mdb_party_similarity": "01_similarities.ipynb",
         "plot_similarity_over_time": "01_similarities.ipynb",
         "align_party_with_party": "01_similarities.ipynb",
         "align_party_with_all_parties": "01_similarities.ipynb",
         "get_party_party_similarity": "01_similarities.ipynb",
         "GUI": "02_gui.ipynb"}

modules = ["parsing.py",
           "similarity.py",
           "gui.py"]

doc_url = "https://eschmidt42.github.io/bundestag/"

git_url = "https://github.com/eschmidt42/bundestag/tree/master/"

def custom_doc_links(name): return None
